package com.usky.fire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.fire.domain.DemFileLabel;
import com.usky.fire.domain.DemFireStatistics;
import com.usky.fire.domain.DemFireStatisticsAttach;
import com.usky.fire.mapper.DemFireStatisticsMapper;
import com.usky.fire.service.DemFileLabelService;
import com.usky.fire.service.DemFireStatisticsAttachService;
import com.usky.fire.service.DemFireStatisticsService;
import com.usky.fire.service.util.OnlineMethod;
import com.usky.fire.service.vo.DemFireStatisticsVo;
import com.usky.fire.service.vo.ParameterVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 火灾统计表 服务实现类
 * </p>
 *
 * @author JCB
 * @since 2022-09-13
 */
@Service
public class DemFireStatisticsServiceImpl extends AbstractCrudService<DemFireStatisticsMapper, DemFireStatistics> implements DemFireStatisticsService {

    @Autowired
    private DemFireStatisticsAttachService demFireStatisticsAttachService;

    @Autowired
    private DemFileLabelService demFileLabelService;

    @Override
    public CommonPage<Object> fireAccidentStatistic(String fireNumber, Integer labelId, Integer classifyId, Integer id, Integer pageNum, Integer pageSize) {
        IPage<DemFireStatistics> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<DemFireStatistics> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(DemFireStatistics::getBurnedArea, DemFireStatistics::getDirectPropertyLoss, DemFireStatistics::getVictimNumber,
                DemFireStatistics::getInjuredNumber, DemFireStatistics::getDeathToll, DemFireStatistics::getFileCase, DemFireStatistics::getFireGrade,
                DemFireStatistics::getSubordinateDetachment, DemFireStatistics::getAdministrativeDivision, DemFireStatistics::getFireNumber,
                DemFireStatistics::getFireAddress, DemFireStatistics::getFireTime, DemFireStatistics::getId, DemFireStatistics::getLabelId,
                DemFireStatistics::getClassifyId)
                .like(StringUtils.isNotBlank(fireNumber), DemFireStatistics::getFireNumber, fireNumber)
                .eq(labelId != null && labelId != 0, DemFireStatistics::getLabelId, labelId)
                .eq(classifyId != null & classifyId != 0, DemFireStatistics::getClassifyId, classifyId)
                .eq(id != null & id != 0, DemFireStatistics::getId, id)
                .orderByDesc(DemFireStatistics::getId);
        page = this.page(page, queryWrapper);
        List<Object> list = new ArrayList<>();
        if (page.getTotal() > 0) {
            List<String> fireNumberList = new ArrayList<>();
            List<Integer> labelIdList = new ArrayList<>();
            for (int i = 0; i < page.getTotal(); i++) {
                fireNumberList.add(page.getRecords().get(i).getFireNumber());
                labelIdList.add(page.getRecords().get(i).getLabelId());
            }
            List<DemFireStatisticsAttach> demFireStatisticsAttaches = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(fireNumberList)) {
                demFireStatisticsAttaches = demFireStatisticsAttachService.fireAccidentStatistic(fireNumberList);
            }

            List<DemFileLabel> demFileLabels = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(labelIdList)) {
                demFileLabels = demFileLabelService.demFileLabelList(null, null, labelIdList);
            }
            for (int i = 0; i < page.getTotal(); i++) {
                Map<String, Object> map = new HashMap<>();
                map.put("id", page.getRecords().get(i).getId());
                map.put("burnedArea", page.getRecords().get(i).getBurnedArea());
                map.put("directPropertyLoss", page.getRecords().get(i).getDirectPropertyLoss());
                map.put("victimNumber", page.getRecords().get(i).getVictimNumber());
                map.put("injuredNumber", page.getRecords().get(i).getInjuredNumber());
                map.put("deathToll", page.getRecords().get(i).getDeathToll());
                map.put("fileCase", page.getRecords().get(i).getFileCase());
                map.put("fireGrade", page.getRecords().get(i).getFireGrade());
                map.put("subordinateDetachment", page.getRecords().get(i).getSubordinateDetachment());
                map.put("administrativeDivision", page.getRecords().get(i).getAdministrativeDivision());
                map.put("fireNumber", page.getRecords().get(i).getFireNumber());
                map.put("fireAddress", page.getRecords().get(i).getFireAddress());
                map.put("fireTime", page.getRecords().get(i).getFireTime());
                map.put("labelId", page.getRecords().get(i).getLabelId());
                map.put("classifyId", page.getRecords().get(i).getClassifyId());
                map.put("zid", null);
                map.put("fireCauseType1", null);
                map.put("fireSiteType1", null);
                map.put("labelName", null);
                for (int j = 0; j < demFireStatisticsAttaches.size(); j++) {
                    if (page.getRecords().get(i).getFireNumber().equals(demFireStatisticsAttaches.get(j).getFireNumber())) {
                        map.put("zid", demFireStatisticsAttaches.get(j).getId());
                        map.put("fireCauseType1", demFireStatisticsAttaches.get(j).getFireCauseType1());
                        map.put("fireSiteType1", demFireStatisticsAttaches.get(j).getFireSiteType1());
                    }
                }
                for (int j = 0; j < demFileLabels.size(); j++) {
                    if (page.getRecords().get(i).getLabelId() == demFileLabels.get(j).getId()) {
                        map.put("labelName", demFileLabels.get(j).getLabelName());
                    }
                }
                list.add(map);
            }
        }
        return new CommonPage<>(list, page.getTotal(), pageSize, pageNum);
    }

    @Transactional
    @Override
    public void addLabel(DemFileLabel demFileLabel) {
        if (demFileLabel.getId() == 0) {
            int count = demFileLabelService.demFileLabelCount(0, demFileLabel.getLabelName(), 0);
            if (count > 0) {
                throw new BusinessException("标签名不可重复");
            }
            demFileLabelService.addDemFileLabel(demFileLabel);
        } else {
            int count = demFileLabelService.demFileLabelCount(0, demFileLabel.getLabelName(), demFileLabel.getId());
            if (count > 0) {
                throw new BusinessException("标签名不可重复");
            }
            demFileLabelService.updateDemFileLabel(demFileLabel);
        }
        int fid = demFileLabel.getId();
        List<DemFileLabel> demFileLabel1 = demFileLabel.getSubset();
        if (CollectionUtils.isNotEmpty(demFileLabel1)) {
            List<DemFileLabel> demFileLabel2 = new ArrayList<>();
            List<DemFileLabel> demFileLabel3 = new ArrayList<>();
            List<String> idList1 = new ArrayList<>();
            List<String> idList2 = new ArrayList<>();
            for (int i = 0; i < demFileLabel1.size(); i++) {
                if (demFileLabel1.get(i).getId() == 0) {
                    demFileLabel2.add(demFileLabel1.get(i));
                } else {
                    demFileLabel3.add(demFileLabel1.get(i));
                    idList1.add(demFileLabel1.get(i).getId() + "");
                }
            }
            List<DemFileLabel> demFileLabels = demFileLabelService.demFileLabelList(fid, null, null);
            if (CollectionUtils.isNotEmpty(demFileLabels)) {
                for (int i = 0; i < demFileLabels.size(); i++) {
                    idList2.add(demFileLabels.get(i).getId() + "");
                }
            }
            List<String> idlist = OnlineMethod.subList2(idList2, idList1);
            if (CollectionUtils.isNotEmpty(idlist)) {
                for (int i = 0; i < idlist.size(); i++) {
                    demFileLabelService.delDemFileLabel(Integer.parseInt(idlist.get(i)));
                }
            }

            if (CollectionUtils.isNotEmpty(demFileLabel3)) {
                for (int i = 0; i < demFileLabel3.size(); i++) {
                    int count = demFileLabelService.demFileLabelCount(fid, demFileLabel3.get(i).getLabelName(), demFileLabel3.get(i).getId());
                    if (count > 0) {
                        throw new BusinessException("分类名不可重复");
                    }
                    demFileLabelService.updateDemFileLabel(demFileLabel3.get(i));
                }
            }

            if (CollectionUtils.isNotEmpty(demFileLabel2)) {
                for (int i = 0; i < demFileLabel2.size(); i++) {
                    demFileLabel2.get(i).setFid(fid);
                    int count = demFileLabelService.demFileLabelCount(fid, demFileLabel2.get(i).getLabelName(), 0);
                    if (count > 0) {
                        throw new BusinessException("分类名不可重复");
                    }
                    demFileLabelService.addDemFileLabel(demFileLabel2.get(i));
                }
            }
        }
    }

    @Override
    public CommonPage<Object> fireStatisticList(String fireNumber, String fireCauseType1, Integer pageNum, Integer pageSize) {
        IPage<DemFireStatisticsAttach> page = new Page<>(pageNum, pageSize);
        LambdaQueryWrapper<DemFireStatisticsAttach> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(DemFireStatisticsAttach::getFireCauseType1, DemFireStatisticsAttach::getFireNumber,
                DemFireStatisticsAttach::getId)
                .like(StringUtils.isNotBlank(fireNumber), DemFireStatisticsAttach::getFireNumber, fireNumber)
                .eq(StringUtils.isNotBlank(fireCauseType1), DemFireStatisticsAttach::getFireCauseType1, fireCauseType1)
                .orderByDesc(DemFireStatisticsAttach::getId);
        page = demFireStatisticsAttachService.page(page, queryWrapper);
        List<Object> list = new ArrayList<>();
        if (page.getTotal() > 0) {
            LambdaQueryWrapper<DemFireStatistics> queryWrapper1 = Wrappers.lambdaQuery();
            queryWrapper1.select(DemFireStatistics::getFireGrade, DemFireStatistics::getAdministrativeDivision,
                    DemFireStatistics::getFireNumber, DemFireStatistics::getFireTime, DemFireStatistics::getId)
                    .like(StringUtils.isNotBlank(fireNumber), DemFireStatistics::getFireNumber, fireNumber)
                    .orderByDesc(DemFireStatistics::getId);
            List<DemFireStatistics> list1 = this.list(queryWrapper1);
            for (int i = 0; i < page.getTotal(); i++) {
                Map<String, Object> map = new HashMap<>();
                map.put("id", null);
                map.put("administrativeDivision", null);
                map.put("fireGrade", null);
                map.put("fireTime", null);
                map.put("zid", page.getRecords().get(i).getId());
                map.put("fireCauseType1", page.getRecords().get(i).getFireCauseType1());
                map.put("fireNumber", page.getRecords().get(i).getFireNumber());
                for (int j = 0; j < list1.size(); j++) {
                    if (page.getRecords().get(i).getFireNumber().equals(list1.get(j).getFireNumber())) {
                        map.put("id", list1.get(j).getId());
                        map.put("fireGrade", list1.get(j).getFireGrade());
                        map.put("administrativeDivision", list1.get(j).getAdministrativeDivision());
                        map.put("fireTime", list1.get(j).getFireTime());
                    }
                }
                list.add(map);
            }
        }
        return new CommonPage<>(list, page.getTotal(), pageSize, pageNum);
    }

    @Transactional
    @Override
    public void bindingLabel(ParameterVo parameterVo) {
        for (int i = 0; i < parameterVo.getIdList().size(); i++) {
            DemFireStatistics demFireStatistics = new DemFireStatistics();
            demFireStatistics.setId(parameterVo.getIdList().get(i));
            demFireStatistics.setLabelId(parameterVo.getLabelId());
            demFireStatistics.setClassifyId(parameterVo.getClassifyId());
            this.updateById(demFireStatistics);
        }
    }


    @Transactional
    @Override
    public void updateDemFireStatistics(DemFireStatisticsVo demFireStatisticsVo) {
        DemFireStatistics demFireStatistics = new DemFireStatistics();
        demFireStatistics.setId(demFireStatisticsVo.getId());
        demFireStatistics.setFireGrade(demFireStatisticsVo.getFireGrade());
        demFireStatistics.setFireAddress(demFireStatisticsVo.getFireAddress());
        demFireStatistics.setBurnedArea(demFireStatisticsVo.getBurnedArea());
        demFireStatistics.setAdministrativeDivision(demFireStatisticsVo.getAdministrativeDivision());
        demFireStatistics.setFillTime(demFireStatisticsVo.getFireTime());
        demFireStatistics.setClassifyId(demFireStatisticsVo.getClassifyId());
        demFireStatistics.setDeathToll(demFireStatisticsVo.getDeathToll());
        demFireStatistics.setFileCase(demFireStatisticsVo.getFileCase());
        demFireStatistics.setDirectPropertyLoss(demFireStatisticsVo.getDirectPropertyLoss());
        demFireStatistics.setLabelId(demFireStatisticsVo.getLabelId());
        demFireStatistics.setInjuredNumber(demFireStatisticsVo.getInjuredNumber());
        demFireStatistics.setSubordinateDetachment(demFireStatisticsVo.getSubordinateDetachment());
        demFireStatistics.setVictimNumber(demFireStatisticsVo.getVictimNumber());
        demFireStatistics.setUpdateTime(LocalDateTime.now());
        demFireStatistics.setUpdatePerson(SecurityUtils.getUsername());
        this.updateById(demFireStatistics);
        DemFireStatisticsAttach demFireStatisticsAttach = new DemFireStatisticsAttach();
        demFireStatisticsAttach.setId(demFireStatisticsVo.getZid());
        demFireStatisticsAttach.setFireCauseType1(demFireStatisticsVo.getFireCauseType1());
        demFireStatisticsAttach.setFireSiteType1(demFireStatisticsVo.getFireSiteType1());
        demFireStatisticsAttach.setUpdateTime(LocalDateTime.now());
        demFireStatisticsAttach.setUpdatePerson(SecurityUtils.getUsername());
        demFireStatisticsAttachService.updateById(demFireStatisticsAttach);
    }

    @Transactional
    @Override
    public void delDemFireStatistics(Integer id, Integer zid) {
        this.removeById(id);
        demFireStatisticsAttachService.removeById(zid);
    }

}
